@@ -11,6 +11,12 @@
 
 	/* mask out mode bits appropriately */
 	mode = mode & ~pseudo_umask;
+        /* if you don't specify a type, assume regular file */
+        if (!(mode & S_IFMT)) {
+                mode |= S_IFREG;
+        }
+        pseudo_debug(PDBGF_FILE, "xmknodat creating '%s', mode 0%o\n",
+                path ? path : "<no name>", (int) mode);
 
 	/* we don't use underlying call, so _ver is irrelevant to us */
 	(void) ver;
@@ -43,6 +49,7 @@
 	if (rc == -1) {
 		return -1;
 	}
+	real_fchmod(rc, PSEUDO_FS_MODE(mode, 0));
 	real___fxstat64(_STAT_VER, rc, &buf);
 	/* mknod does not really open the file.  We don't have
 	 * to use wrap_close because we've never exposed this file
