@@ -7,7 +7,7 @@
  *	char * rc = NULL;
  */
 
-	pseudo_debug(3, "getwd (getcwd)\n");
+	pseudo_debug(PDBGF_CLIENT, "getwd (getcwd)\n");
 	rc = wrap_getcwd(buf, pseudo_path_max());
 	/* because it would violate everything we have ever known about
 	 * UNIX for these functions to have the same errno semantics,
