@@ -37,8 +37,6 @@
         rc = real_link(oldpath, newpath);
         save_errno = errno;
         if (rc == -1) {
-                free(oldpath);
-                free(newpath);
                 errno = save_errno;
                 return rc;
         }
@@ -53,13 +51,11 @@
         if (rc2 == -1) {
                 pseudo_diag("Fatal: Tried to stat '%s' after linking it, but failed: %s.\n",
                         oldpath, strerror(errno));
-                free(oldpath);
-                free(newpath);
                 errno = ENOENT;
                 return rc;
         }
         msg = pseudo_client_op(OP_STAT, 0, -1, -1, oldpath, &buf);
-        if (msg) {
+        if (msg && msg->result == RESULT_SUCCEED) {
                 pseudo_stat_msg(&buf, msg);
         }
         /* Long story short: I am pretty sure we still want OP_LINK even
@@ -67,8 +63,6 @@
          */
         pseudo_client_op(OP_LINK, 0, -1, -1, newpath, &buf);
 
-        free(oldpath);
-        free(newpath);
         errno = save_errno;
 
 /*	return rc;
