@@ -19,7 +19,7 @@
  */
 #include <stdlib.h>
 #include <ctype.h>
-#include <stdlib.h>
+#include <signal.h>
 #include <string.h>
 #include <stdio.h>
 #include <errno.h>
@@ -85,51 +85,40 @@ usage(int status) {
 	exit(status);
 }
 
-/* helper function to make a directory, just like mkdir -p.
- * Can't use system() because the child shell would end up trying
- * to do the same thing...
- */
-static void
-mkdir_p(char *path) {
-	size_t len = strlen(path);
-	size_t i;
-
-	for (i = 1; i < len; ++i) {
-		/* try to create all the directories in path, ignoring
-		 * failures
-		 */
-		if (path[i] == '/') {
-			path[i] = '\0';
-			(void) mkdir(path, 0755);
-			path[i] = '/';
-		}
-	}
-	(void) mkdir(path, 0755);
-}
 
 /* main server process */
 int
 main(int argc, char *argv[]) {
 	int o;
 	char *s;
-	int lockfd, newfd;
 	char *ld_env = getenv(PRELINK_LIBRARIES);
 	int rc = 0;
 	char opts[pseudo_path_max()], *optptr = opts;
-	char *lockname;
-	char *lockpath;
+	sigset_t blocked, saved;
 
 	opts[0] = '\0';
 
 	pseudo_init_util();
 
+	/* The pseudo client will have blocked these, and sigprocmask
+	 * is inherited, but we want them to work.
+	 */
+	sigemptyset(&blocked);
+	sigaddset(&blocked, SIGALRM);	/* every-N-seconds tasks */
+	sigaddset(&blocked, SIGCHLD);	/* reaping child processes */
+	sigaddset(&blocked, SIGHUP);	/* idiomatically, reloading config */
+	sigaddset(&blocked, SIGTERM);	/* shutdown/teardown operations */
+	sigaddset(&blocked, SIGUSR1);	/* reopening log files, sometimes */
+	sigaddset(&blocked, SIGUSR2);	/* who knows what people do */
+	sigprocmask(SIG_UNBLOCK, &blocked, &saved);
+
 	if (ld_env && strstr(ld_env, "libpseudo")) {
-		pseudo_debug(PDBGF_SERVER, "can't run daemon with libpseudo in %s\n", PRELINK_LIBRARIES);
+		pseudo_debug(PDBGF_SERVER, "[server %d] can't run daemon with libpseudo in %s\n", getpid(), PRELINK_LIBRARIES);
 		s = pseudo_get_value("PSEUDO_UNLOAD");
 		if (s) {
 			pseudo_diag("pseudo: I can't seem to make %s go away.  Sorry.\n", PRELINK_LIBRARIES);
 			pseudo_diag("pseudo: %s: %s\n", PRELINK_LIBRARIES, ld_env);
-			exit(EXIT_FAILURE);
+			exit(PSEUDO_EXIT_PSEUDO_LOADED);
 		}
 		free(s);
 		pseudo_set_value("PSEUDO_UNLOAD", "YES");
@@ -137,7 +126,7 @@ main(int argc, char *argv[]) {
 		pseudo_dropenv(); /* Drop PRELINK_LIBRARIES */
 
 		execv(argv[0], argv);
-		exit(EXIT_FAILURE);
+		exit(PSEUDO_EXIT_PSEUDO_LOADED);
 	}
 
 	/* Be sure to clean PSEUDO_UNLOAD so if we're asked to run any
@@ -269,7 +258,7 @@ main(int argc, char *argv[]) {
 
 	if (!pseudo_get_prefix(argv[0])) {
 		pseudo_diag("Can't figure out prefix.  Set PSEUDO_PREFIX or invoke with full path.\n");
-		exit(EXIT_FAILURE);
+		exit(PSEUDO_EXIT_PSEUDO_PREFIX);
 	}
 
 	/* move database */
@@ -319,7 +308,7 @@ main(int argc, char *argv[]) {
 			pseudo_diag("Couldn't allocate data structure for path.\n");
 			exit(EXIT_FAILURE);
 		}
-		if (pdb_find_file_path(msg, NULL)) {
+		if (pdb_find_file_path(msg)) {
 			pseudo_diag("Couldn't find a database entry for '%s'.\n", opt_i);
 			exit(EXIT_FAILURE);
 		}
@@ -345,18 +334,18 @@ main(int argc, char *argv[]) {
 
         /* If you didn't specify a command, opt_S shuts down here. */
 	if (opt_S && argc <= optind) {
-		return pseudo_client_shutdown();
+		return pseudo_client_shutdown(0);
 	}
 
 	if (opt_d && opt_f) {
 		pseudo_diag("You cannot run a foregrounded daemon.\n");
-		exit(EXIT_FAILURE);
+		exit(PSEUDO_EXIT_PSEUDO_INVOCATION);
 	}
 
 	if (opt_f || opt_d) {
 		if (argc > optind) {
 			pseudo_diag("pseudo: running program implies spawning background daemon.\n");
-			exit(EXIT_FAILURE);
+			exit(PSEUDO_EXIT_PSEUDO_INVOCATION);
 		}
 	} else {
 		char fullpath[pseudo_path_max()];
@@ -420,7 +409,7 @@ main(int argc, char *argv[]) {
 			 * the server.
 			 */
                         if (opt_S) {
-				pseudo_client_shutdown();
+				pseudo_client_shutdown(1);
                         }
 			if (WIFEXITED(rc)) {
 				return WEXITSTATUS(rc);
@@ -439,54 +428,6 @@ main(int argc, char *argv[]) {
 			exit(EXIT_FAILURE);
 		}
 	}
-	/* if we got here, we are not running a command, and we are not in
-	 * a pseudo environment.
-	 */
-	pseudo_new_pid();
-
-	pseudo_debug(PDBGF_SERVER, "opening lock.\n");
-	lockpath = pseudo_localstatedir_path(NULL);
-	if (!lockpath) {
-		pseudo_diag("Couldn't allocate a file path.\n");
-		exit(EXIT_FAILURE);
-	}
-	mkdir_p(lockpath);
-	lockname = pseudo_localstatedir_path(PSEUDO_LOCKFILE);
-	if (!lockname) {
-		pseudo_diag("Couldn't allocate a file path.\n");
-		exit(EXIT_FAILURE);
-	}
-	lockfd = open(lockname, O_RDWR | O_CREAT, 0644);
-	if (lockfd < 0) {
-		pseudo_diag("Can't open or create lockfile %s: %s\n",
-			lockname, strerror(errno));
-		exit(EXIT_FAILURE);
-	}
-	free(lockname);
-
-	if (lockfd <= 2) {
-		newfd = fcntl(lockfd, F_DUPFD, 3);
-		if (newfd < 0) {
-			pseudo_diag("Can't move lockfile to safe descriptor: %s\n",
-				strerror(errno));
-		} else {
-			close(lockfd);
-			lockfd = newfd;
-		}
-	}
-
-	pseudo_debug(PDBGF_SERVER, "acquiring lock.\n");
-	if (flock(lockfd, LOCK_EX | LOCK_NB) < 0) {
-		if (errno == EACCES || errno == EAGAIN) {
-			pseudo_debug(PDBGF_SERVER, "Existing server has lock.  Exiting.\n");
-		} else {
-			pseudo_diag("pseudo: Error obtaining lock: %s\n", strerror(errno));
-		}
-		exit(0);
-	} else {
-		pseudo_debug(PDBGF_SERVER, "Acquired lock.\n");
-	}
-	pseudo_debug(PDBGF_SERVER, "serving (%s)\n", opt_d ? "daemon" : "foreground");
 	return pseudo_server_start(opt_d);
 }
 
@@ -499,7 +440,6 @@ int
 pseudo_op(pseudo_msg_t *msg, const char *program, const char *tag, char **response_path, size_t *response_len) {
 	pseudo_msg_t msg_header;
 	pseudo_msg_t by_path = { .op = 0 }, by_ino = { .op = 0 };
-	long long row = -1;
 	pseudo_msg_t db_header;
 	char *path_by_ino = 0;
 	char *oldpath = 0;
@@ -543,6 +483,7 @@ pseudo_op(pseudo_msg_t *msg, const char *program, const char *tag, char **respon
 		case OP_GET_XATTR:
 		case OP_LIST_XATTR:
 		case OP_REPLACE_XATTR:
+		case OP_REMOVE_XATTR:
 		case OP_SET_XATTR:
 			/* In a rename there are two paths, null separated in msg->path */
 			initial_len = strlen(msg->path);
@@ -595,7 +536,7 @@ pseudo_op(pseudo_msg_t *msg, const char *program, const char *tag, char **respon
 	 */
 	/* Lookup the full path, with inode and dev if available */
 	if (msg->pathlen && msg->dev && msg->ino) {
-		if (!pdb_find_file_exact(msg, &row)) {
+		if (!pdb_find_file_exact(msg)) {
 			/* restore header contents */
 			by_path = *msg;
 			by_ino = *msg;
@@ -614,7 +555,7 @@ pseudo_op(pseudo_msg_t *msg, const char *program, const char *tag, char **respon
 		if (msg->pathlen) {
 			/* for now, don't canonicalize paths anymore */
 			/* used to do it here, but now doing it in client */
-			if (!pdb_find_file_path(msg, &row)) {
+			if (!pdb_find_file_path(msg)) {
 				by_path = *msg;
 				found_path = 1;
 			} else {
@@ -625,7 +566,7 @@ pseudo_op(pseudo_msg_t *msg, const char *program, const char *tag, char **respon
 		}
 		/* search on original inode -- in case of mismatch */
 		if (msg->dev && msg->ino) {
-			if (!pdb_find_file_dev(&by_ino, &row, &path_by_ino)) {
+			if (!pdb_find_file_dev(&by_ino, &path_by_ino)) {
 				found_ino = 1;
 			}
 		}
@@ -659,7 +600,12 @@ pseudo_op(pseudo_msg_t *msg, const char *program, const char *tag, char **respon
 				if (by_path.deleting != 0) {
 					pseudo_debug(PDBGF_FILE, "inode mismatch for '%s' -- old one was marked for deletion, deleting.\n",
 						msg->path);
-					pdb_did_unlink_file(msg->path, by_path.deleting);
+					/* in this case, we don't trust the
+					 * existing entries, so we will do the
+					 * more expensive sweep for stray
+					 * xattrs.
+					 */
+					pdb_did_unlink_file(msg->path, NULL, by_path.deleting);
 				} else {
 					pseudo_diag("inode mismatch: '%s' ino %llu in db, %llu in request.\n",
 						msg->path,
@@ -757,7 +703,7 @@ pseudo_op(pseudo_msg_t *msg, const char *program, const char *tag, char **respon
 				if (by_ino.deleting != 0) {
 					pseudo_debug(PDBGF_FILE, "inode mismatch for '%s' -- old one was marked for deletion, deleting.\n",
 						msg->path);
-					pdb_did_unlink_file(path_by_ino, by_ino.deleting);
+					pdb_did_unlink_file(path_by_ino, &by_ino, by_ino.deleting);
 				} else {
 					pseudo_diag("path mismatch [%d link%s]: ino %llu db '%s' req '%s'.\n",
 						msg->nlink,
@@ -835,7 +781,7 @@ pseudo_op(pseudo_msg_t *msg, const char *program, const char *tag, char **respon
 		if (!found_path) {
 			pseudo_debug(PDBGF_DB, "linking %s for OP_CREAT\n",
 				msg->pathlen ? msg->path : "no path");
-			pdb_link_file(msg, NULL);
+			pdb_link_file(msg);
 		} else {
 			/* again, an error, but leaving it alone for now. */
 			pseudo_diag("creat ignored for existing file '%s'.\n",
@@ -870,7 +816,7 @@ pseudo_op(pseudo_msg_t *msg, const char *program, const char *tag, char **respon
 			pseudo_debug(PDBGF_FILE, "(new) ");
 			pseudo_debug(PDBGF_DB, "linking %s for OP_[F]CHMOD\n",
 				msg->pathlen ? msg->path : "no path");
-			pdb_link_file(msg, NULL);
+			pdb_link_file(msg);
 		}
 		break;
 	case OP_CHOWN:		/* FALLTHROUGH */
@@ -902,7 +848,7 @@ pseudo_op(pseudo_msg_t *msg, const char *program, const char *tag, char **respon
 			pseudo_debug(PDBGF_FILE, "(new) ");
 			pseudo_debug(PDBGF_DB, "linking %s for OP_[F]CHOWN\n",
 				msg->pathlen ? msg->path : "no path");
-			pdb_link_file(msg, NULL);
+			pdb_link_file(msg);
 		}
 		break;
 	case OP_STAT:		/* FALLTHROUGH */
@@ -972,7 +918,7 @@ pseudo_op(pseudo_msg_t *msg, const char *program, const char *tag, char **respon
 		pseudo_debug(PDBGF_DB, "linking %s for %s\n",
 			msg->pathlen ? msg->path : "no path",
 			pseudo_op_name(msg->op));
-		pdb_link_file(msg, NULL);
+		pdb_link_file(msg);
 		break;
 	case OP_RENAME:
 		/* a rename implies renaming an existing entry... and every
@@ -989,7 +935,7 @@ pseudo_op(pseudo_msg_t *msg, const char *program, const char *tag, char **respon
 		}
 		break;
 	case OP_DID_UNLINK:
-		pdb_did_unlink_file(msg->path, msg->client);
+		pdb_did_unlink_file(msg->path, msg, msg->client);
 		break;
 	case OP_CANCEL_UNLINK:
 		pdb_cancel_unlink_file(msg);
@@ -1037,10 +983,10 @@ pseudo_op(pseudo_msg_t *msg, const char *program, const char *tag, char **respon
 		pseudo_debug(PDBGF_DB, "linking %s for %s\n",
 			msg->pathlen ? msg->path : "no path",
 			pseudo_op_name(msg->op));
-		pdb_link_file(msg, NULL);
+		pdb_link_file(msg);
 		break;
 	case OP_GET_XATTR:
-		if (pdb_get_xattr(row, &oldpath, &oldpathlen)) {
+		if (pdb_get_xattr(msg, &oldpath, &oldpathlen)) {
 			msg->result = RESULT_FAIL;
 		} else {
 			*response_path = oldpath;
@@ -1050,7 +996,7 @@ pseudo_op(pseudo_msg_t *msg, const char *program, const char *tag, char **respon
 		}
 		break;
 	case OP_LIST_XATTR:
-		if (pdb_list_xattr(row, &oldpath, &oldpathlen)) {
+		if (pdb_list_xattr(msg, &oldpath, &oldpathlen)) {
 			msg->result = RESULT_FAIL;
 		} else {
 			pseudo_debug(PDBGF_XATTR, "got %d bytes of xattrs to list: %.*s\n", (int) oldpathlen, (int) oldpathlen, oldpath);
@@ -1067,28 +1013,12 @@ pseudo_op(pseudo_msg_t *msg, const char *program, const char *tag, char **respon
 			xattr_flags = XATTR_REPLACE;
 		}
 	case OP_SET_XATTR:
-		/* we need a row entry to store xattr info */
-		if (row == -1) {
-#ifdef PSEUDO_XATTRDB
-			/* mark the entry as Not Reliable for purposes
-			 * of stat-type calls, since changes wouldn't
-			 * get reported to us.
-			 */
-			msg->uid = (uid_t) -1;
-			msg->gid = (gid_t) -1;
-#endif
-			pseudo_debug(PDBGF_DB | PDBGF_XATTR, "linking %s (uid -1 if xattr) for %s, mode 0%o\n",
-				msg->pathlen ? msg->path : "no path",
-				pseudo_op_name(msg->op),
-                                (int) msg->mode);
-			pdb_link_file(msg, &row);
-		}
-		if (pdb_set_xattr(row, oldpath, oldpathlen, xattr_flags)) {
+		if (pdb_set_xattr(msg, oldpath, oldpathlen, xattr_flags)) {
 			msg->result = RESULT_FAIL;
 		}
 		break;
 	case OP_REMOVE_XATTR:
-		pdb_remove_xattr(row, oldpath, oldpathlen);
+		pdb_remove_xattr(msg, oldpath, oldpathlen);
 		break;
 	default:
 		pseudo_diag("unknown op from client %d, op %d [%s]\n",
