@@ -105,6 +105,9 @@ gid_t pseudo_egid;
 gid_t pseudo_sgid;
 gid_t pseudo_fgid;
 
+int (*pseudo_real_fork)(void) = fork;
+int (*pseudo_real_execv)(const char *, char * const *) = execv;
+
 #define PSEUDO_ETC_FILE(filename, realname, flags) pseudo_etc_file(filename, realname, flags, passwd_paths, npasswd_paths)
 
 /* helper function to make a directory, just like mkdir -p.
@@ -939,32 +942,46 @@ client_spawn_server(void) {
 	FILE *fp;
 	char * pseudo_pidfile;
 
-	if ((server_pid = fork()) != 0) {
+	if ((server_pid = pseudo_real_fork()) != 0) {
 		if (server_pid == -1) {
 			pseudo_diag("couldn't fork server: %s\n", strerror(errno));
 			return 1;
 		}
+		pseudo_evlog(PDBGF_CLIENT, "spawned new server, pid %d\n", server_pid);
 		pseudo_debug(PDBGF_CLIENT | PDBGF_SERVER, "spawned server, pid %d\n", server_pid);
 		/* wait for the child process to terminate, indicating server
 		 * is ready
 		 */
 		waitpid(server_pid, &status, 0);
+		if (WIFEXITED(status)) {
+			pseudo_evlog(PDBGF_CLIENT, "server exited status %d\n", WEXITSTATUS(status));
+		}
+		if (WIFSIGNALED(status)) {
+			pseudo_evlog(PDBGF_CLIENT, "server exited from signal %d\n", WTERMSIG(status));
+		}
 		server_pid = -2;
-		pseudo_pidfile = pseudo_localstatedir_path(PSEUDO_PIDFILE);
-		fp = fopen(pseudo_pidfile, "r");
-		if (fp) {
-			if (fscanf(fp, "%d", &server_pid) != 1) {
-				pseudo_debug(PDBGF_CLIENT, "Opened server PID file, but didn't get a pid.\n");
+		if (WIFEXITED(status) && WEXITSTATUS(status) == 0) {
+			pseudo_evlog(PDBGF_CLIENT, "server reports successful startup, reading pid.\n");
+			pseudo_pidfile = pseudo_localstatedir_path(PSEUDO_PIDFILE);
+			fp = fopen(pseudo_pidfile, "r");
+			if (fp) {
+				if (fscanf(fp, "%d", &server_pid) != 1) {
+					pseudo_debug(PDBGF_CLIENT, "Opened server PID file, but didn't get a pid.\n");
+				}
+				fclose(fp);
+			} else {
+				pseudo_debug(PDBGF_CLIENT, "no pid file (%s): %s\n",
+					pseudo_pidfile, strerror(errno));
 			}
-			fclose(fp);
+			pseudo_debug(PDBGF_CLIENT, "read new pid file: %d\n", server_pid);
+			free(pseudo_pidfile);
+			/* at this point, we should have a new server_pid */
+			return 0;
 		} else {
-			pseudo_debug(PDBGF_CLIENT, "no pid file (%s): %s\n",
-				pseudo_pidfile, strerror(errno));
+			pseudo_evlog(PDBGF_CLIENT, "server startup apparently unsuccessful.  setting server pid to -1.\n");
+			server_pid = -1;
+			return 1;
 		}
-		pseudo_debug(PDBGF_CLIENT, "read new pid file: %d\n", server_pid);
-		free(pseudo_pidfile);
-		/* at this point, we should have a new server_pid */
-		return 0;
 	} else {
 		char *base_args[] = { NULL, NULL, NULL };
 		char **argv;
@@ -1018,14 +1035,22 @@ client_spawn_server(void) {
 				close(fd);
 		}
 		/* and now, execute the server */
-
-		pseudo_set_value("PSEUDO_UNLOAD", "YES");
-		pseudo_setupenv();
-		pseudo_dropenv(); /* drop PRELINK_LIBRARIES */
-
 		pseudo_debug(PDBGF_CLIENT | PDBGF_SERVER | PDBGF_INVOKE, "calling execv on %s\n", argv[0]);
 
-		execv(argv[0], argv);
+		/* don't try to log this exec, because it'll cause the process
+		 * that is supposed to be spawning the server to try to spawn
+		 * a server. Whoops. This is because the exec wrapper doesn't
+		 * respect antimagic, which I believe is intentional.
+		 */
+		pseudo_client_logging = 0;
+
+		/* manual setup of environment, so we can call real-execv
+		 * instead of the wrapper.
+		 */
+		pseudo_set_value("PSEUDO_UNLOAD", "YES");
+		pseudo_setupenv();
+		pseudo_dropenv();
+		pseudo_real_execv(argv[0], argv);
 		pseudo_diag("critical failure: exec of pseudo daemon failed: %s\n", strerror(errno));
 		exit(1);
 	}
@@ -1122,8 +1147,11 @@ client_connect(void) {
 
 	connect_fd = socket(PF_UNIX, SOCK_STREAM, 0);
 	connect_fd = pseudo_fd(connect_fd, MOVE_FD);
+	pseudo_evlog(PDBGF_CLIENT, "creating socket %s.\n", sun.sun_path);
 	if (connect_fd == -1) {
-		pseudo_diag("Can't create socket: %s (%s)\n", sun.sun_path, strerror(errno));
+		char *e = strerror(errno);
+		pseudo_diag("Can't create socket: %s (%s)\n", sun.sun_path, e);
+		pseudo_evlog(PDBGF_CLIENT, "failed to create socket: %s\n", e);
 		return 1;
 	}
 
@@ -1145,7 +1173,9 @@ client_connect(void) {
 		return 1;
 	}
 	if (connect(connect_fd, (struct sockaddr *) &sun, sizeof(sun)) == -1) {
-		pseudo_debug(PDBGF_CLIENT, "Can't connect socket to pseudo.socket: (%s)\n", strerror(errno));
+		char *e = strerror(errno);
+		pseudo_debug(PDBGF_CLIENT, "Can't connect socket to pseudo.socket: (%s)\n", e);
+		pseudo_evlog(PDBGF_CLIENT, "connect failed: %s\n", e);
 		close(connect_fd);
 		if (fchdir(cwd_fd) == -1) {
 			pseudo_diag("return to previous directory failed: %s\n",
@@ -1160,6 +1190,7 @@ client_connect(void) {
 			strerror(errno));
 	}
 	close(cwd_fd);
+	pseudo_evlog(PDBGF_CLIENT, "socket connect OK.\n");
 	pseudo_debug(PDBGF_CLIENT | PDBGF_VERBOSE, "connected socket.\n");
 	return 0;
 }
@@ -1175,51 +1206,93 @@ pseudo_client_setup(void) {
 		close(connect_fd);
 		connect_fd = -1;
 	}
+	if (client_connect() == 0) {
+		pseudo_debug(PDBGF_CLIENT | PDBGF_VERBOSE, "connection started.\n");
+		if (client_ping() == 0) {
+			pseudo_debug(PDBGF_CLIENT, "connection ping okay, server ready.\n");
+			return 0;
+		} else {
+			pseudo_debug(PDBGF_CLIENT, "connection up, but ping failed.\n");
+			/* and now we'll close the connection, and try to
+			 * kick the server.
+			 */
+			close(connect_fd);
+			connect_fd = -1;
+		}
+	}
+	pseudo_debug(PDBGF_CLIENT, "connection failed, checking server...\n");
 
+	/* This whole section is purely informational; we want to know
+	 * what might be up with the server, but we're going to try to
+	 * start one anyway.
+	 */
 	pseudo_pidfile = pseudo_localstatedir_path(PSEUDO_PIDFILE);
 	fp = fopen(pseudo_pidfile, "r");
+	free(pseudo_pidfile);
 	if (fp) {
 		if (fscanf(fp, "%d", &server_pid) != 1) {
 			pseudo_debug(PDBGF_CLIENT, "Opened server PID file, but didn't get a pid.\n");
 		}
 		fclose(fp);
 	}
-	if (server_pid) {
+	if (server_pid > 0) {
 		if (kill(server_pid, 0) == -1) {
 			pseudo_debug(PDBGF_CLIENT, "couldn't find server at pid %d: %s\n",
 				server_pid, strerror(errno));
 			server_pid = 0;
+		} else {
+			pseudo_debug(PDBGF_CLIENT, "server pid should be %d, which exists, but connection failed.\n",
+				server_pid);
+			/* and we'll restart the server anyway. */
 		}
+	} else {
+		pseudo_debug(PDBGF_CLIENT, "no server pid available.\n");
 	}
-	if (!server_pid) {
-		if (client_spawn_server()) {
-			return 1;
-		}
-	}
-	if (!client_connect() && !client_ping()) {
-		return 0;
-	}
-	pseudo_debug(PDBGF_CLIENT, "server seems to be gone, trying to restart\n");
 	if (client_spawn_server()) {
 
+		pseudo_evlog(PDBGF_CLIENT, "attempted respawn, failed.\n");
 		pseudo_debug(PDBGF_CLIENT, "failed to spawn server, waiting for retry.\n");
 		return 1;
 	} else {
-		pseudo_debug(PDBGF_CLIENT, "restarted, new pid %d\n", server_pid);
-		if (!client_connect() && !client_ping()) {
-			return 0;
+		pseudo_evlog(PDBGF_CLIENT, "restarted, new pid %d, will retry message\n", server_pid);
+		pseudo_debug(PDBGF_CLIENT, "restarted, new pid %d, will retry message\n", server_pid);
+		/* so we think a server has started. Now we'll retry the
+		 * connect/ping, because if they work we'll have set
+		 * connect_fd correctly, and will have succeeded.
+		 */
+		if (!client_connect()) {
+			pseudo_debug(PDBGF_CLIENT, "connect okay to restarted server.\n");
+			pseudo_evlog(PDBGF_CLIENT, "connect okay to restarted server.\n");
+			if (!client_ping()) {
+				pseudo_debug(PDBGF_CLIENT, "ping okay to restarted server.\n");
+				pseudo_evlog(PDBGF_CLIENT, "ping okay to restarted server.\n");
+				return 0;
+			} else {
+				pseudo_debug(PDBGF_CLIENT, "ping failed to restarted server.\n");
+				pseudo_evlog(PDBGF_CLIENT, "ping failed to restarted server.\n");
+				close(connect_fd);
+				connect_fd = -1;
+				return 1;
+			}
+		} else {
+			pseudo_debug(PDBGF_CLIENT, "connect failed to restarted server.\n");
+			pseudo_evlog(PDBGF_CLIENT, "connect failed to restarted server.\n");
+			return 1;
 		}
 	}
-	pseudo_debug(PDBGF_CLIENT, "couldn't get or spawn a server.\n");
-	return 1;
 }
 
-#define PSEUDO_RETRIES 50
+#define PSEUDO_RETRIES 20
 static pseudo_msg_t *
 pseudo_client_request(pseudo_msg_t *msg, size_t len, const char *path) {
 	pseudo_msg_t *response = 0;
 	int tries = 0;
 	int rc;
+	extern char *program_invocation_short_name;
+	#if 0
+	if (!strcmp(program_invocation_short_name, "pseudo"))
+		abort();
+	#endif
 
 	if (!msg)
 		return 0;
@@ -1229,11 +1302,13 @@ pseudo_client_request(pseudo_msg_t *msg, size_t len, const char *path) {
 	 * It's okay if we can't start a server sometimes, because another
 	 * client may have done it.
 	 */
+	pseudo_debug(PDBGF_CLIENT | PDBGF_VERBOSE, "sending a message: ino %llu\n",
+		(unsigned long long) msg->ino);
         for (tries = 0; tries < PSEUDO_RETRIES; ++tries) {
-		pseudo_debug(PDBGF_CLIENT | PDBGF_VERBOSE, "sending a message: ino %llu\n",
-			(unsigned long long) msg->ino);
+		pseudo_evlog(PDBGF_CLIENT, "try %d, connect fd is %d\n", tries, connect_fd);
 		rc = pseudo_msg_send(connect_fd, msg, len, path);
 		if (rc != 0) {
+			pseudo_evlog(PDBGF_CLIENT, "msg_send failed [%d].\n", rc);
 			pseudo_debug(PDBGF_CLIENT | PDBGF_VERBOSE, "msg_send: %d%s\n",
 				rc,
 				rc == -1 ? " (sigpipe)" :
@@ -1243,9 +1318,12 @@ pseudo_client_request(pseudo_msg_t *msg, size_t len, const char *path) {
 			 * retry.
 			 */
 			if (pseudo_client_setup()) {
-				int ms = (getpid() % 5) + 3 + tries;
+				int ms = (getpid() % 5) + (3 * tries);
 				struct timespec delay = { .tv_sec = 0, .tv_nsec = ms * 1000000 };
+				pseudo_evlog(PDBGF_CLIENT, "setup failed, delaying %d ms.\n", ms);
 				nanosleep(&delay, NULL);
+			} else {
+				pseudo_evlog(PDBGF_CLIENT, "setup apparently successful, retrying message immediately.\n");
 			}
 			continue;
 		}
@@ -1271,12 +1349,15 @@ pseudo_client_request(pseudo_msg_t *msg, size_t len, const char *path) {
 		}
 	}
 	pseudo_diag("pseudo: server connection persistently failed, aborting.\n");
+	pseudo_evlog_dump();
+	pseudo_diag("event log dumped, aborting.\n");
 	abort();
+	pseudo_diag("aborted.\n");
 	return 0;
 }
 
 int
-pseudo_client_shutdown(void) {
+pseudo_client_shutdown(int wait_on_socket) {
 	pseudo_msg_t msg;
 	pseudo_msg_t *ack;
 	char *pseudo_path;
@@ -1347,12 +1428,17 @@ pseudo_client_shutdown(void) {
 		pseudo_diag("server did not respond to shutdown query.\n");
 		return 1;
 	}
-	if (ack->type == PSEUDO_MSG_ACK) {
-		return 0;
+	if (ack->type != PSEUDO_MSG_ACK) {
+		pseudo_diag("Server refused shutdown.  Remaining client fds: %d\n", ack->fd);
+		pseudo_diag("Client pids: %s\n", ack->path);
+		pseudo_diag("Server will shut down after all clients exit.\n");
+	}
+	if (wait_on_socket) {
+		/* try to receive a message the server won't send;
+		 * this should abort/error-out when the server actually
+		 * shuts down. */
+		ack = pseudo_msg_receive(connect_fd);
 	}
-	pseudo_diag("Server refused shutdown.  Remaining client fds: %d\n", ack->fd);
-	pseudo_diag("Client pids: %s\n", ack->path);
-	pseudo_diag("Server will shut down after all clients exit.\n");
 	return 0;
 }
 
@@ -1595,7 +1681,7 @@ pseudo_client_op(pseudo_op_t op, int access, int fd, int dirfd, const char *path
 	 * empty path for that.
 	 */
 	if (path_extra_1) {
-		size_t full_len = path_extra_1len + 1 + pathlen;
+		size_t full_len = path_extra_1len + 1 + pathlen - strip_slash;
 		size_t partial_len = pathlen - 1 - strip_slash;
 		if (path_extra_2) {
 			full_len += path_extra_2len + 1;
@@ -1735,7 +1821,7 @@ pseudo_client_op(pseudo_op_t op, int access, int fd, int dirfd, const char *path
 		break;
 	case OP_DUP:
 		/* just copy the path over */
-		pseudo_debug(PDBGF_CLIENT, "dup: fd_path(%d) = %p [%s], dup to %d\n",
+		pseudo_debug(PDBGF_CLIENT | PDBGF_VERBOSE, "dup: fd_path(%d) = %p [%s], dup to %d\n",
 			fd, fd_path(fd), fd_path(fd) ? fd_path(fd) : "<nil>",
 			dirfd);
 		pseudo_client_path(dirfd, fd_path(fd));
