@@ -72,11 +72,11 @@ typedef struct { char *fmt; int arg; } id_row;
  */
 static id_row files_default_migrations[] = {
 	{ "%d, -1, 'N/A (initial setup)'", 8 },
-	{ NULL },
+	{ NULL, 0 },
 };
 static id_row logs_default_migrations[] = {
 	{ "%d, -1, 'N/A (initial setup)'", 7 },
-	{ NULL },
+	{ NULL, 0 },
 };
 /* This seemed like a really good idea at the time.  The idea is that these
  * structures let me write semi-abstract code to "create a database" without
@@ -135,7 +135,7 @@ static struct sql_table {
 	    "gid INTEGER, "
 	    "access INTEGER, "
 	    "program VARCHAR, "
-	    "type INTEGER, "
+	    "type INTEGER"
 	    ,
 	  NULL,
 	  NULL },
@@ -1486,7 +1486,6 @@ pdb_clear_unused_xattrs(void) {
 		dberr(file_db, "delete of unused xattrs may have failed");
 	}
 	sqlite3_reset(delete);
-	sqlite3_clear_bindings(delete);
 }
 
 /* we want to delete extended attributes for a device/inode if there
@@ -1496,7 +1495,7 @@ pdb_clear_unused_xattrs(void) {
 static void
 pdb_clear_xattrs(pseudo_msg_t *msg) {
 	static sqlite3_stmt *delete;
-	char *delete_sql = "DELETE FROM xattrs WHERE dev = ? AND ino = ?;";
+	char *delete_sql = "DELETE FROM xattrs WHERE dev = ? AND ino = ? AND (SELECT COUNT(*) FROM files WHERE dev = ? AND ino = ?) = 0;";
 	int rc;
 
 	if (!msg)
@@ -1508,12 +1507,14 @@ pdb_clear_xattrs(pseudo_msg_t *msg) {
 	if (!delete) {
 		rc = sqlite3_prepare_v2(file_db, delete_sql, strlen(delete_sql), &delete, NULL);
 		if (rc) {
-			dberr(file_db, "couldn't prepare DELETE statement for unused xattrs");
+			dberr(file_db, "couldn't prepare DELETE statement for specific inode xattrs");
 			return;
 		}
 	}
 	sqlite3_bind_int(delete, 1, msg->dev);
 	sqlite3_bind_int(delete, 2, msg->ino);
+	sqlite3_bind_int(delete, 3, msg->dev);
+	sqlite3_bind_int(delete, 4, msg->ino);
 	rc = sqlite3_step(delete);
 	if (rc != SQLITE_DONE) {
 		dberr(file_db, "delete of unused xattrs may have failed");
@@ -1595,10 +1596,6 @@ pdb_check_xattrs(pseudo_msg_t *msg) {
 	if (rc != SQLITE_DONE) {
 		dberr(file_db, "not done after the single row we expected?", rc);
 	}
-	rc = sqlite3_finalize(scan);
-	if (rc) {
-		dberr(file_db, "couldn't finalize existing file");
-	}
 	sqlite3_reset(scan);
 	sqlite3_clear_bindings(scan);
 
@@ -1852,7 +1849,7 @@ pdb_did_unlink_files(int deleting) {
 
 /* confirm deletion of a specific file by a given client */
 int
-pdb_did_unlink_file(char *path, int deleting) {
+pdb_did_unlink_file(char *path, pseudo_msg_t *msg, int deleting) {
 	static sqlite3_stmt *delete_exact;
 	int rc, exact;
 	char *sql_delete_exact = "DELETE FROM files WHERE path = ? AND deleting = ?;";
@@ -1882,11 +1879,14 @@ pdb_did_unlink_file(char *path, int deleting) {
 	exact = sqlite3_changes(file_db);
 	pseudo_debug(PDBGF_DB, "(exact %d)\n", exact);
 	sqlite3_reset(delete_exact);
-	sqlite3_clear_bindings(delete_exact);
-	/* we have to clean everything because we don't know for sure the
-	 * device/inode...
-	 */
-	pdb_clear_unused_xattrs();
+	if (msg) {
+		pdb_clear_xattrs(msg);
+	} else {
+		/* we have to clean everything because we don't know for sure the
+		 * device/inode...
+		 */
+		pdb_clear_unused_xattrs();
+	}
 	return rc != SQLITE_DONE;
 }
 
